/***
*
*	Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "soundent.h"
#include "gamerules.h"

enum GrenadeLauncher_e
{
	GrenadeLauncher_LONGIDLE = 0,
	GrenadeLauncher_IDLE1,
	GrenadeLauncher_LAUNCH,
	GrenadeLauncher_RELOAD,
	GrenadeLauncher_DEPLOY,
	GrenadeLauncher_FIRE1,
	GrenadeLauncher_FIRE2,
	GrenadeLauncher_FIRE3,
};



LINK_ENTITY_TO_CLASS( weapon_GrenadeLauncher, CGrenadeLauncher );
//LINK_ENTITY_TO_CLASS( weapon_9mmAR, CMP5 );


//=========================================================
//=========================================================
int CGrenadeLauncher::SecondaryAmmoIndex( void )
{
	return m_iSecondaryAmmoType;
}

void CGrenadeLauncher::Spawn( )
{
	pev->classname = MAKE_STRING("weapon_9mmAR"); // hack to allow for old names
	Precache( );
	SET_MODEL(ENT(pev), "models/w_9mmAR.mdl");
	m_iId = WEAPON_GrenadeLauncher;

	m_iDefaultAmmo = GrenadeLauncher_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}


void CGrenadeLauncher::Precache( void )
{
	PRECACHE_MODEL("models/v_rpg.mdl");
	PRECACHE_MODEL("models/w_rpg.mdl");
	PRECACHE_MODEL("models/p_rpg.mdl");

	m_iShell = PRECACHE_MODEL ("models/w_grenade.mdl");// brass shellTE_MODEL

	m_usGrenadeLauncher = PRECACHE_EVENT( 1, "events/GrenadeLauncher.sc" );

	PRECACHE_MODEL("models/w_grenade.mdl");	// grenade

	PRECACHE_MODEL("models/w_grenade.mdl");
	PRECACHE_SOUND("items/9mmclip1.wav");              

	PRECACHE_SOUND("items/clipinsert1.wav");
	PRECACHE_SOUND("items/cliprelease1.wav");

	PRECACHE_SOUND ("weapons/hks1.wav");// H to the K
	PRECACHE_SOUND ("weapons/hks2.wav");// H to the K
	PRECACHE_SOUND ("weapons/hks3.wav");// H to the K

	PRECACHE_SOUND( "weapons/glauncher.wav" );
	PRECACHE_SOUND( "weapons/glauncher2.wav" );

	PRECACHE_SOUND ("weapons/357_cock1.wav");


	
}

int CGrenadeLauncher::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "Hand Grenade";
	p->iMaxAmmo1 = GrenadeLauncher_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = RPG_MAX_CLIP;
	p->iSlot = 5;
	p->iPosition = 4;
	p->iFlags = 0;
	p->iId = m_iId = WEAPON_GrenadeLauncher;
	p->iWeight = RPG_WEIGHT;

	return 1;
}

int CGrenadeLauncher::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CGrenadeLauncher::Deploy( )
{
	return DefaultDeploy( "models/v_rpg.mdl", "models/p_rpg.mdl", GrenadeLauncher_DEPLOY, "GrenadeLauncher" );
}


void CGrenadeLauncher::PrimaryAttack()
{
		// don't fire underwater
	if (m_pPlayer->pev->waterlevel == 3)
	{
		PlayEmptySound( );
		m_flNextPrimaryAttack = 0.15;
		return;
	}

	if (m_pPlayer->m_rgAmmo[m_iSecondaryAmmoType] == 0)
	{
		PlayEmptySound( );
		return;
	}

	m_pPlayer->m_iWeaponVolume = NORMAL_GUN_VOLUME;
	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;

	m_pPlayer->m_iExtraSoundTypes = bits_SOUND_DANGER;
	m_pPlayer->m_flStopExtraSoundTime = UTIL_WeaponTimeBase() + 0.2;
			
	m_pPlayer->m_rgAmmo[m_iSecondaryAmmoType]--;

	// player "shoot" animation
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

 	UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );

	// we don't add in player velocity anymore.
	CGrenade::ShootContact( m_pPlayer->pev, 
							m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs + gpGlobals->v_forward * 16, 
							gpGlobals->v_forward * 800 );

	int flags;
#if defined( CLIENT_WEAPONS )
	flags = FEV_NOTHOST;
#else
	flags = 0;
#endif

	PLAYBACK_EVENT( flags, m_pPlayer->edict(), m_usGrenadeLauncher );
	
	m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 1;
	m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1;
	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 5;// idle pretty soon after shooting.

	if (!m_pPlayer->m_rgAmmo[m_iSecondaryAmmoType])
		// HEV suit - indicate out of ammo condition
		m_pPlayer->SetSuitUpdate("!HEV_AMO0", FALSE, 0);}



void CGrenadeLauncher::SecondaryAttack( void )
{

}

void CGrenadeLauncher::Reload( void )
{
	if ( m_pPlayer->ammo_9mm <= 0 )
		return;

	DefaultReload( GrenadeLauncher_MAX_CLIP, GrenadeLauncher_RELOAD, 1.5 );
}


void CGrenadeLauncher::WeaponIdle( void )
{
	ResetEmptySound( );

	m_pPlayer->GetAutoaimVector( AUTOAIM_5DEGREES );

	if ( m_flTimeWeaponIdle > UTIL_WeaponTimeBase() )
		return;

	int iAnim;
	switch ( RANDOM_LONG( 0, 1 ) )
	{
	case 0:	
		iAnim = GrenadeLauncher_LONGIDLE;	
		break;
	
	default:
	case 1:
		iAnim = GrenadeLauncher_IDLE1;
		break;
	}

	SendWeaponAnim( iAnim );

	m_flTimeWeaponIdle = UTIL_SharedRandomFloat( m_pPlayer->random_seed, 10, 15 ); // how long till we do this again.
}



class CGrenadeLauncherAmmoClip : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		//SET_MODEL(ENT(pev), "models/w_9mmARclip.mdl");
		SET_MODEL(ENT(pev), "models/w_grenade.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_grenade.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int bResult = (pOther->GiveAmmo( AMMO_RPGCLIP_GIVE, "Hand Grenade", GrenadeLauncher_MAX_CARRY) != -1);
		if (bResult)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
		}
		return bResult;
	}
};
LINK_ENTITY_TO_CLASS( ammo_grenadelauncherclip, CGrenadeLauncherAmmoClip );
//LINK_ENTITY_TO_CLASS( ammo_9mmAR, CMP5AmmoClip );





